import { Component, Input, ViewChild, Output, EventEmitter, NgModule, InjectionToken, Injectable, Inject, defineInjectable, inject } from '@angular/core';
import { CommonModule } from '@angular/common';
import { forkJoin, Subject } from 'rxjs';
import { HttpClientModule, HttpRequest, HttpClient, HttpEventType, HttpResponse } from '@angular/common/http';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class Ng7LargeFilesUploadLibComponent {
    /**
     * @param {?} service
     */
    constructor(service) {
        this.service = service;
        /**
         * if true the component will be responsible to send data at the specified endpoint
         */
        this.canSendData = false;
        /**
         * Allows to download attachments
         */
        this.canDownload = true;
        /**
         * Upload new attachments behavior
         */
        this.canUpload = true;
        /**
         * Delete attachments uploaded behavior
         */
        this.canDelete = true;
        /**
         * Text for save button if canSendData = true
         */
        this.saveButtonText = "Guardar";
        /**
         * Initial attachments
         */
        this._attachments = [];
        //Attachments added
        this._newAttachments = [];
        //Attachments wich already have
        this._oldAttachments = [];
        /**
         * Return the list of attachments in each change
         */
        this.updatedAttachmentsEmitter = new EventEmitter();
        this.files = new Array();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        //Clone initial attachments array
        this._oldAttachments = JSON.parse(JSON.stringify(this._attachments));
    }
    // public _newAttachments: Set<File> = new Set();
    /**
     * @return {?}
     */
    addFiles() {
        this.file.nativeElement.click();
    }
    /**
     * @param {?} f
     * @param {?} aArray
     * @return {?}
     */
    removeFiles(f, aArray) {
        this.deleteFromArray(f, aArray);
        this.removeFromAllAttachments(f);
    }
    /**
     * @param {?} f
     * @return {?}
     */
    deleteFiles(f) {
        /** @type {?} */
        let a = this._attachments.find((/**
         * @param {?} x
         * @return {?}
         */
        x => x.id == f.id));
        a.isDelete = true;
        a.isInsert = false;
        this.deleteFromArray(a, this._oldAttachments);
    }
    /**
     * @param {?} f
     * @param {?} aArray
     * @return {?}
     */
    deleteFromArray(f, aArray) {
        aArray.forEach((/**
         * @param {?} item
         * @param {?} index
         * @return {?}
         */
        (item, index) => {
            if ((item.fileName === f.name || item.name === f.name))
                aArray.splice(index, 1);
        }));
        this.updatedAttachmentsEmitter.emit(this._attachments);
    }
    /**
     * @param {?} f
     * @return {?}
     */
    removeFromAllAttachments(f) {
        this._attachments.forEach((/**
         * @param {?} item
         * @param {?} index
         * @return {?}
         */
        (item, index) => {
            if (item === f)
                this._attachments.splice(index, 1);
        }));
        this.updatedAttachmentsEmitter.emit(this._attachments);
    }
    /**
     * @return {?}
     */
    onFilesAdded() {
        Array.prototype.forEach.call(this.file.nativeElement.files, (/**
         * @param {?} f
         * @return {?}
         */
        (f) => {
            /** @type {?} */
            let newAttach = this._newAttachments.find((/**
             * @param {?} x
             * @return {?}
             */
            x => x.name == f.name));
            /** @type {?} */
            let oldAttach = this._oldAttachments.find((/**
             * @param {?} x
             * @return {?}
             */
            x => x.name == f.name));
            /** @type {?} */
            let attach = this._attachments.find((/**
             * @param {?} x
             * @return {?}
             */
            x => x.name == f.name));
            /** @type {?} */
            let attachNew = this._attachments.find((/**
             * @param {?} x
             * @return {?}
             */
            x => x.name == f.name && x.id == undefined));
            if (newAttach) {
                this.deleteFromArray(newAttach, this._newAttachments);
                if (attachNew) {
                    this.removeFromAllAttachments(attachNew);
                }
            }
            else if (oldAttach) {
                this.deleteFiles(attach);
                this.deleteFromArray(oldAttach, this._oldAttachments);
            }
            f.isInsert = true;
            f.isDelete = false;
            this.files.push(f);
            this._newAttachments.push(f);
            this._attachments.push(f);
            this.updatedAttachmentsEmitter.emit(this._attachments);
        }));
    }
    /**
     * @return {?}
     */
    onSaveChanges() {
        if (!this.canSendData)
            return;
        /** @type {?} */
        let uploadStatus = this.service.upload(this._attachments);
        /** @type {?} */
        let allProgressObservables = [];
        for (let key in uploadStatus) {
            allProgressObservables.push(uploadStatus[key].progress);
        }
        if (allProgressObservables != undefined) {
            forkJoin(allProgressObservables).subscribe((/**
             * @param {?} end
             * @return {?}
             */
            end => {
                console.log("all finished");
            }));
        }
    }
}
Ng7LargeFilesUploadLibComponent.decorators = [
    { type: Component, args: [{
                selector: 'mb-large-files-uploader',
                template: "<div>\r\n  <input type=\"file\" #file style=\"display: none\" (change)=\"onFilesAdded()\" multiple />\r\n  <div class=\"evidences-icon-container\">\r\n    <span>{{title}}</span>\r\n    <i *ngIf=\"canUpload\" class=\"material-icons add-file-icon\" (click)=\"addFiles()\">add_circle</i>\r\n  </div>\r\n  <div *ngIf=\"canDownload || canDelete\">\r\n    <div class=\"file-container\" *ngFor=\"let oa of _oldAttachments\">\r\n      <a *ngIf=\"canDownload\" href=\"{{oa.url}}\">\r\n        <i class=\"material-icons file-icon download-file-icon\">vertical_align_bottom</i>\r\n      </a>\r\n      <i *ngIf=\"canDelete\" class=\"material-icons file-icon delete-file-icon\" (click)=\"deleteFiles(oa)\">delete</i>\r\n      <span>{{oa.name}}</span>\r\n    </div>\r\n  </div>\r\n  <div *ngIf=\"canUpload\">\r\n    <div class=\"file-container\" *ngFor=\"let na of _newAttachments\">\r\n      <i class=\"material-icons file-icon remove-file-icon\" (click)=\"removeFiles(na, _newAttachments)\">remove_circle</i>\r\n      <span>{{na.name}}</span>\r\n    </div>\r\n  </div>\r\n</div>\r\n<button *ngIf=\"canSendData\" (click)=\"onSaveChanges()\">{{saveButtonText}}</button>",
                styles: [".attachments-title{font-size:16px}.add-file-icon{color:#397fbb;cursor:pointer;vertical-align:bottom;margin-left:15px}.file-icon{cursor:pointer;vertical-align:bottom;margin-left:15px;margin-right:15px}.remove-file-icon{color:#d62121}.download-file-icon{color:#000}.delete-file-icon{color:red}.evidences-container{margin-bottom:40px}.evidences-icon-container{margin-bottom:20px}.file-container{margin-top:10px}div{font-family:Roboto}@font-face{font-family:Roboto;font-style:normal;font-weight:500;src:local('Roboto Medium'),local('Roboto-Medium'),url(https://fonts.gstatic.com/s/roboto/v18/KFOlCnqEu92Fr1MmEU9fChc4EsA.woff2) format('woff2');unicode-range:U+0100-024F,U+0259,U+1E00-1EFF,U+2020,U+20A0-20AB,U+20AD-20CF,U+2113,U+2C60-2C7F,U+A720-A7FF}@font-face{font-family:'Material Icons';font-style:normal;font-weight:400;src:url(https://fonts.gstatic.com/s/materialicons/v43/flUhRq6tzZclQEJ-Vdg-IuiaDsNc.woff2) format('woff2')}.material-icons{font-family:'Material Icons';font-weight:400;font-style:normal;font-size:24px;line-height:1;letter-spacing:normal;text-transform:none;display:inline-block;white-space:nowrap;word-wrap:normal;direction:ltr;-webkit-font-feature-settings:'liga';-webkit-font-smoothing:antialiased}"]
            }] }
];
/** @nocollapse */
Ng7LargeFilesUploadLibComponent.ctorParameters = () => [
    { type: Ng7LargeFilesUploadLibService }
];
Ng7LargeFilesUploadLibComponent.propDecorators = {
    file: [{ type: ViewChild, args: ['file',] }],
    canSendData: [{ type: Input }],
    canDownload: [{ type: Input }],
    canUpload: [{ type: Input }],
    canDelete: [{ type: Input }],
    title: [{ type: Input }],
    saveButtonText: [{ type: Input }],
    _attachments: [{ type: Input }],
    updatedAttachmentsEmitter: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const BASE_URL = new InjectionToken('BASE_URL');
class Ng7LargeFilesUploadLibModule {
    /**
     * @param {?} host
     * @return {?}
     */
    static forRoot(host) {
        return {
            ngModule: Ng7LargeFilesUploadLibModule,
            providers: [{
                    provide: BASE_URL,
                    useValue: host
                }]
        };
    }
}
Ng7LargeFilesUploadLibModule.decorators = [
    { type: NgModule, args: [{
                declarations: [Ng7LargeFilesUploadLibComponent],
                imports: [
                    CommonModule,
                    HttpClientModule,
                ],
                exports: [Ng7LargeFilesUploadLibComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class Ng7LargeFilesUploadLibService {
    /**
     * @param {?} base
     * @param {?} _httpClient
     */
    constructor(base, _httpClient) {
        this._httpClient = _httpClient;
        this.BASE_URL = base;
    }
    /**
     * @param {?} files
     * @return {?}
     */
    upload(files) {
        debugger;
        /** @type {?} */
        const status = {};
        /** @type {?} */
        const url = this.BASE_URL;
        files.forEach((/**
         * @param {?} file
         * @return {?}
         */
        file => {
            /** @type {?} */
            const formData = new FormData();
            formData.append('file', file, file.name);
            /** @type {?} */
            const req = new HttpRequest('POST', url, formData, {
                reportProgress: true
            });
            /** @type {?} */
            const progress = new Subject();
            this._httpClient.request(req).subscribe((/**
             * @param {?} event
             * @return {?}
             */
            event => {
                if (event.type === HttpEventType.UploadProgress) {
                    /** @type {?} */
                    const percentDone = Math.round(100 * event.loaded / event.total);
                    progress.next(percentDone);
                }
                else if (event instanceof HttpResponse) {
                    progress.complete();
                }
            }));
            status[file.name] = {
                progress: progress.asObservable()
            };
        }));
        return status;
    }
}
Ng7LargeFilesUploadLibService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
Ng7LargeFilesUploadLibService.ctorParameters = () => [
    { type: String, decorators: [{ type: Inject, args: [BASE_URL,] }] },
    { type: HttpClient }
];
/** @nocollapse */ Ng7LargeFilesUploadLibService.ngInjectableDef = defineInjectable({ factory: function Ng7LargeFilesUploadLibService_Factory() { return new Ng7LargeFilesUploadLibService(inject(BASE_URL), inject(HttpClient)); }, token: Ng7LargeFilesUploadLibService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { Ng7LargeFilesUploadLibService, Ng7LargeFilesUploadLibComponent, BASE_URL, Ng7LargeFilesUploadLibModule };

//# sourceMappingURL=ng7-large-files-upload-lib.js.map